package com.yihu.wlyy.util;

import com.yihu.wlyy.util.httpUtil.HttpResponse;
import com.yihu.wlyy.util.httpUtil.HttpUtils;
import org.apache.log4j.Logger;
import org.apache.commons.net.ftp.*;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by zdm on 2019/5/7.
 */
@Component
public class FtpFilesUtil {
    static Logger Log = Logger.getLogger(FtpFilesUtil.class);
    private FTPClient ftpClient;
    private static String ENCODING = "GBK";
    FTPClientConfig ftpConfig = new FTPClientConfig("UNIX");
    @Value("${wlyy.url}")
    private String wlyyUrl;
    @Value("${file.default_path}")
    private String fileDefaultPath;


    /**
     * 连接到ftp
     * @param address ip
     * @param port 端口
     * @param name 用户名
     * @param passwd 密码
     * @return
     */
    public boolean connectServer(String address,int port,String name,String passwd) {
        boolean flag = false;
        try {
            ftpClient = new FTPClient();
            ftpConfig.setServerLanguageCode("ISO-8859-1");
            ftpClient.connect(address, port);
            if(ftpClient.login(name,passwd)){
                if (FTPReply.isPositiveCompletion(ftpClient.sendCommand("OPTS UTF8", "ON"))) {
                    ENCODING = "UTF-8";
                }
                ftpClient.setControlEncoding(ENCODING);
                ftpClient.enterLocalPassiveMode();
                ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
                ftpClient.setFileTransferMode(FTP.STREAM_TRANSFER_MODE);
                ftpClient.setDataTimeout(120000);
            }
            int reply = ftpClient.getReplyCode();
            if (FTPReply.isPositiveCompletion(reply)) {
                flag = true;
                Log.info("FTP connect success!");
            } else {
                Log.warn("FTP refused to connect!");
                ftpClient.disconnect();
            }
        } catch (Exception e) {
            Log.error("Failed to login ftp " + address + ":" + port, e);
        }
        return flag;
    }

    /**
     * 判断服务器上是否存在该文件
     * @param path
     * @return
     * @throws IOException
     */
    public boolean existFile(String path) throws IOException {
        boolean flag = false;
        FTPFile[] ftpFileArr = ftpClient.listFiles(path);
        if (ftpFileArr.length > 0) {
            flag = true;
        }else {
            Log.info("file ‘"+path+ "' does not exist ! ");
        }
        return flag;
    }

    /**
     * 上传文件到服务器
     * @param remoteFilePath 远程文件路径： /data/rec2/2019/0507/08/52.mp3
     * @param uploadFile 待上传文件： D:\testFile\52.mp3
     * @return
     * @throws IOException
     */
    public boolean uploadFile(String remoteFilePath, File uploadFile) throws IOException {
        boolean flag = false;
        InputStream input = null;
        try {
            input = new FileInputStream(uploadFile);
            String remote = new String(remoteFilePath.getBytes(ENCODING), "ISO-8859-1");
            if (ftpClient.appendFile(remote, input)) {
                flag = true;
            }
        } finally {
            input.close();
        }
        Log.info("push file (" + uploadFile.getCanonicalPath() + ") => " + (flag ? "SUCCESS" : "FAILED"));
        return flag;
    }

    /**
     * 关闭服务器连接
     */
    public void closeConnect() {
        try {
            if (ftpClient != null) {
                ftpClient.logout();
                ftpClient.disconnect();
                Log.info("FTP connect closed!");
            }
        } catch (Exception e) {
            Log.info("Failed to close FTP connection!");
            e.printStackTrace();
        }
    }

    /**
     * 下载文件
     * @param remoteFilePath 远程文件：/data/rec2/2019/0507/08/52.mp3
     * @param downloadFile 下载到本地的文件：D:\testFile\52.mp3
     * @return
     * @throws IOException
     */
    public boolean downloadFile(String remoteFilePath, File downloadFile) throws IOException {
        boolean flag = false;
        OutputStream out = null;
        InputStream in=null;
        try {
            //获取文件流
            in = ftpClient.retrieveFileStream(remoteFilePath);
            // 文件读取方式一
            out = new FileOutputStream(downloadFile);
            int i;
            byte[] bytes = new byte[1024];
            while ((i = in.read(bytes)) != -1) {
                out.write(bytes, 0, i);
            }
            out.flush();
        }catch (IOException e){
            Log.info("File download failed :"+e.getMessage());
            e.printStackTrace();
        }finally {
            try {
                if(null!=in){
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                if(null!=in){
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        Log.info("File Download Successful");
        return flag;
    }

    /**
     * 递归遍历出目录下面所有文件
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public JSONArray getFileListByPath(String url,int port,String user,String pwd,String pathName) throws Exception {
        JSONArray jsonArray = new JSONArray();
        //连接ftp服务器
        connectServer(url,port,user,pwd);
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            ftpClient.enterLocalPassiveMode();
            ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = ftpClient.listFiles();
            JSONObject jsonObject;
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        jsonObject = new JSONObject();
                        jsonObject.put("fileName",files[i].getName());
                        jsonArray.put(jsonObject);
                    }
                }
            }
        }
        //关闭连接
        closeConnect();
        return jsonArray;
    }

    /**
     * 从FTP下载文件到本地
     * @param ftpClient     已经登陆成功的FTPClient
     * @param ftpFilePath   FTP上的目标文件路径
     * @param localFilePath 下载到本地的文件路径
     */
    public void downloadFileFromFTP(FTPClient ftpClient, String ftpFilePath, String localFilePath,String fileName) {
        InputStream is = null;
        FileOutputStream fos = null;
        try {
            // 获取ftp上的文件
            is = ftpClient.retrieveFileStream(ftpFilePath);
            fos = new FileOutputStream(new File(localFilePath+fileName));
            // 文件读取方式一
            int i;
            byte[] bytes = new byte[1024];
            while ((i = is.read(bytes)) != -1) {
                fos.write(bytes, 0, i);
            }
            ftpClient.completePendingCommand();
            Log.info("FTP文件下载成功！");
        } catch (Exception e) {
            Log.error("FTP文件下载失败！", e);
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 递归遍历出目录下面所有文件
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public JSONArray getFileListByPathTest(String url,int port,String user,String pwd,String pathName) throws Exception {
        JSONArray jsonArray = new JSONArray();
        //连接ftp服务器
        connectServer(url,port,user,pwd);
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            ftpClient.enterLocalPassiveMode();
            ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = ftpClient.listFiles();
            JSONObject jsonObject;
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        downloadFileFromFTP(ftpClient, pathName+files[i].getName(),"D:\\testFile\\", files[i].getName());

                    }
                }
            }
        }
        //关闭连接
        closeConnect();
        return jsonArray;
    }
    //========================================================文件获取并上传=================================================================================================

    /**
     * 递归遍历出目录下面所有文件
     * @param url ftp_ip
     * @param port ftp端口
     * @param user ftp登录用户名
     * @param pwd ftp密码
     * @throws Exception
     */
    public void uploadFtpFileJobByDate(String url, int port, String user, String pwd) throws Exception {
        //连接ftp服务器
        connectServer(url, port, user, pwd);
        //获取今天日期 拼凑录音文件在ftp上的路径
        String year=DateUtil.getNowYear()+"";
        String date=   DateUtil.dateToStr(DateUtil.getNowDate(),DateUtil.MMDD);
        String pathName=fileDefaultPath+year+"/"+date+"/";
        ergodicFiles(pathName);
        //关闭连接
        closeConnect();
    }

    public void ergodicFiles(String pathName) throws Exception{
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            this.ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            this.ftpClient.enterLocalPassiveMode();
            this.ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = this.ftpClient.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        byte[] byt = getFileStreamFromFTP(this.ftpClient, pathName + files[i].getName());
                        Map<String, Object> params = new HashMap<>();
                        String str = Base64Utils.encode(byt);
                        params.put("name", files[i].getName());
                        params.put("bytes", str);
                        Log.info("DATE：" + DateUtil.getStringDate() + ";FILES_NAME=" + files[i].getName());
                        HttpResponse res = HttpUtils.doPost(wlyyUrl + "upload/ftpVoiceUpload", params);
                        JSONObject rs = new JSONObject(res.getContent());
                        Log.info("res=" + rs.get("data").toString() + "");
                    }else{
                        String time=files[i].getName();
                        Log.info("filesPathName===================="+pathName+time+"/");
                        ergodicFiles(pathName+time+"/");
                    }
                }
            }
        }
    }

    /**
     * 递归遍历出目录下面所有文件
     * @param pathName 需要遍历的目录，必须以"/"开始和结束
     * @throws IOException
     */
    public void uploadFtpFileByPath(String url, int port, String user, String pwd, String pathName) throws Exception {
        //连接ftp服务器
        connectServer(url, port, user, pwd);
        if (pathName.startsWith("/") && pathName.endsWith("/")) {
            //更换目录到当前目录
            ftpClient.changeWorkingDirectory(pathName);
            //设置访问被动模式
            ftpClient.enterLocalPassiveMode();
            ftpClient.setRemoteVerificationEnabled(false);
            //检索ftp目录下所有的文件，利用时间字符串进行过滤
            FTPFile[] files = ftpClient.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        byte[] byt = getFileStreamFromFTP(ftpClient, pathName + files[i].getName());
                        Map<String, Object> params = new HashMap<>();
                        String str = Base64Utils.encode(byt);
                        params.put("name", files[i].getName());
                        params.put("bytes", str);
                        Log.info("DATE_" + DateUtil.getNowDate() + ";FILES___NAME=" + files[i].getName());
                        HttpResponse res = HttpUtils.doPost(wlyyUrl + "upload/ftpVoiceUpload", params);
                        JSONObject rs = new JSONObject(res.getContent());
                        Log.info("res=" + rs.get("data").toString() + "");
                    }
                }
            }
        }
        //关闭连接
        closeConnect();
    }


    /**
     * 从FTP下载文件到本地
     * @param ftpClient     已经登陆成功的FTPClient
     * @param ftpFilePath   FTP上的目标文件路径
     */
    public byte[] getFileStreamFromFTP(FTPClient ftpClient, String ftpFilePath) {
        InputStream is = null;
        try {
            // 获取ftp上的文件
            is = ftpClient.retrieveFileStream(ftpFilePath);
            byte[] byt = new byte[is.available()];
            is.read(byt);
            Log.info("FTP文件下载成功！");
            return byt;
        } catch (Exception e) {
            Log.error("FTP文件下载失败！", e);
        } finally {
            try {
                if (is != null) {
                    is.close();
                    ftpClient.completePendingCommand();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }



}
